RankedError

Enum RankedError 

pub enum RankedError<R, E>
where R: TransientError<E>,
{ Transient(R), Error(E), }
An error type consisting of a transient (non-critical) error and an unignorable error.

Variants



Transient(R)

This error can be ignored if the allowed by the caller code. Such errors should be acknowledged (see TransientError::acknowledge before being dropped, but aren’t necessarily required to be.



Error(E)

This error cannot be ignored and should be propagated. The caller should never suppress values in this alternative.