pub enum ElementStatus {
Valid,
Deleted,
}Expand description
Describe the status of accessing a vector by internal or external id.
Variants
Valid
The ID is valid.
Deleted
The ID used to be valid but points to a deleted element. Some values behind the deleted may still be accessible until it is removed entirely from the graph.
Implementations
Sourceimpl ElementStatus
impl ElementStatus