Module utils

Module utils 

Re-exports

pub use vector_repr::VectorRepr;
pub use utils::*;

Modules

async_tools
utils
vector_repr

Traits

VectorId
The data type used to assign an identity to vectors. Canonically, this is either a u32 or a u64. Since this type is used to stored neighbor information in graphs, using u32 can reduce the memory footprint significantly and should be preferred if possible. However, DiskANN supports both types.