Module config
diskann
0.54.0
Module config
Module Items
Modules
Structs
Enums
In diskann::
graph
diskann
::
graph
Module
config
Modules

defaults
experimental
Structs

Builder
A builder for for
Config
. Necessary invariants among the fields will be checked upon invoking [
Config::build
].
Config
Configuration state for index construction operations.
Config
Error
Errors that can occur when building a
Config
.
Enums

Intra
Batch
Candidates
Controls the number of edges considered among an insert batch for the multi-insert API.
MaxDegree
Configuration setting for the maximum graph degree.
Prune
Kind
The occlusion factor between a vector
i
and an candidate neighbor
k
tracks how close the vectors
i
and
k
are relative to all other vectors and are directly comparable to the build parameter
alpha
.