Re-exports
pub use vector_repr::VectorRepr;pub use utils::*;
Modules
Traits
- Vector
Id - The data type used to assign an identity to vectors. Canonically, this is either a
u32or au64. Since this type is used to stored neighbor information in graphs, usingu32can reduce the memory footprint significantly and should be preferred if possible. However, DiskANN supports both types.